"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from opperai import errors, models, utils
from opperai._hooks import HookContext
from opperai.entries import Entries
from opperai.types import OptionalNullable, UNSET
from opperai.utils import get_security_from_env
from typing import Any, List, Mapping, Optional


class Datasets(BaseSDK):
    entries: Entries

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.entries = Entries(self.sdk_configuration)

    def create_entry(
        self,
        *,
        dataset_id: str,
        input: Any,
        output: Any,
        expected: OptionalNullable[Any] = UNSET,
        comment: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateDatasetEntryResponse:
        r"""Create Dataset Entry

        :param dataset_id: The id of the dataset
        :param input: The input to the dataset entry
        :param output: The output to the dataset entry
        :param expected: The expected output to the dataset entry, this is an optionally edited version of the output
        :param comment: The comment to the dataset entry
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateDatasetEntryDatasetsDatasetIDPostRequest(
            dataset_id=dataset_id,
            create_dataset_entry_request=models.CreateDatasetEntryRequest(
                input=input,
                output=output,
                expected=expected,
                comment=comment,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/datasets/{dataset_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_dataset_entry_request,
                False,
                False,
                "json",
                models.CreateDatasetEntryRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_dataset_entry_datasets__dataset_id__post",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.CreateDatasetEntryResponse
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def create_entry_async(
        self,
        *,
        dataset_id: str,
        input: Any,
        output: Any,
        expected: OptionalNullable[Any] = UNSET,
        comment: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateDatasetEntryResponse:
        r"""Create Dataset Entry

        :param dataset_id: The id of the dataset
        :param input: The input to the dataset entry
        :param output: The output to the dataset entry
        :param expected: The expected output to the dataset entry, this is an optionally edited version of the output
        :param comment: The comment to the dataset entry
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateDatasetEntryDatasetsDatasetIDPostRequest(
            dataset_id=dataset_id,
            create_dataset_entry_request=models.CreateDatasetEntryRequest(
                input=input,
                output=output,
                expected=expected,
                comment=comment,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/datasets/{dataset_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_dataset_entry_request,
                False,
                False,
                "json",
                models.CreateDatasetEntryRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_dataset_entry_datasets__dataset_id__post",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.CreateDatasetEntryResponse
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def list_entries(
        self,
        *,
        dataset_id: str,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PaginatedResponseGetDatasetEntriesResponse:
        r"""List Dataset Entries

        :param dataset_id: The id of the dataset
        :param offset: The offset of the entries to get
        :param limit: The limit of the entries to get
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListDatasetEntriesDatasetsDatasetIDEntriesGetRequest(
            dataset_id=dataset_id,
            offset=offset,
            limit=limit,
        )

        req = self._build_request(
            method="GET",
            path="/datasets/{dataset_id}/entries",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list_dataset_entries_datasets__dataset_id__entries_get",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.PaginatedResponseGetDatasetEntriesResponse
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def list_entries_async(
        self,
        *,
        dataset_id: str,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PaginatedResponseGetDatasetEntriesResponse:
        r"""List Dataset Entries

        :param dataset_id: The id of the dataset
        :param offset: The offset of the entries to get
        :param limit: The limit of the entries to get
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListDatasetEntriesDatasetsDatasetIDEntriesGetRequest(
            dataset_id=dataset_id,
            offset=offset,
            limit=limit,
        )

        req = self._build_request_async(
            method="GET",
            path="/datasets/{dataset_id}/entries",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list_dataset_entries_datasets__dataset_id__entries_get",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.PaginatedResponseGetDatasetEntriesResponse
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get_entry(
        self,
        *,
        dataset_id: str,
        entry_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetDatasetEntryResponse:
        r"""Get Dataset Entry

        :param dataset_id: The id of the dataset
        :param entry_id: The id of the entry to get
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetDatasetEntryDatasetsDatasetIDEntriesEntryIDGetRequest(
            dataset_id=dataset_id,
            entry_id=entry_id,
        )

        req = self._build_request(
            method="GET",
            path="/datasets/{dataset_id}/entries/{entry_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_dataset_entry_datasets__dataset_id__entries__entry_id__get",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.GetDatasetEntryResponse)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_entry_async(
        self,
        *,
        dataset_id: str,
        entry_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetDatasetEntryResponse:
        r"""Get Dataset Entry

        :param dataset_id: The id of the dataset
        :param entry_id: The id of the entry to get
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetDatasetEntryDatasetsDatasetIDEntriesEntryIDGetRequest(
            dataset_id=dataset_id,
            entry_id=entry_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/datasets/{dataset_id}/entries/{entry_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_dataset_entry_datasets__dataset_id__entries__entry_id__get",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.GetDatasetEntryResponse)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def delete_entry(
        self,
        *,
        dataset_id: str,
        entry_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete Dataset Entry

        :param dataset_id: The id of the dataset
        :param entry_id: The id of the entry to delete
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteDatasetEntryDatasetsDatasetIDEntriesEntryIDDeleteRequest(
            dataset_id=dataset_id,
            entry_id=entry_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/datasets/{dataset_id}/entries/{entry_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="delete_dataset_entry_datasets__dataset_id__entries__entry_id__delete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def delete_entry_async(
        self,
        *,
        dataset_id: str,
        entry_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete Dataset Entry

        :param dataset_id: The id of the dataset
        :param entry_id: The id of the entry to delete
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteDatasetEntryDatasetsDatasetIDEntriesEntryIDDeleteRequest(
            dataset_id=dataset_id,
            entry_id=entry_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/datasets/{dataset_id}/entries/{entry_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="delete_dataset_entry_datasets__dataset_id__entries__entry_id__delete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def query_entries(
        self,
        *,
        dataset_id: str,
        query: str,
        limit: Optional[int] = 5,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.QueryDatasetEntriesResponse]:
        r"""Query Dataset Entries

        :param dataset_id: The id of the dataset
        :param query: The query to search for
        :param limit: The limit of the entries to get
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.QueryDatasetEntriesDatasetsDatasetIDEntriesQueryPostRequest(
            dataset_id=dataset_id,
            query=query,
            limit=limit,
        )

        req = self._build_request(
            method="POST",
            path="/datasets/{dataset_id}/entries/query",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="query_dataset_entries_datasets__dataset_id__entries_query_post",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, List[models.QueryDatasetEntriesResponse]
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def query_entries_async(
        self,
        *,
        dataset_id: str,
        query: str,
        limit: Optional[int] = 5,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.QueryDatasetEntriesResponse]:
        r"""Query Dataset Entries

        :param dataset_id: The id of the dataset
        :param query: The query to search for
        :param limit: The limit of the entries to get
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.QueryDatasetEntriesDatasetsDatasetIDEntriesQueryPostRequest(
            dataset_id=dataset_id,
            query=query,
            limit=limit,
        )

        req = self._build_request_async(
            method="POST",
            path="/datasets/{dataset_id}/entries/query",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="query_dataset_entries_datasets__dataset_id__entries_query_post",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, List[models.QueryDatasetEntriesResponse]
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .updatespanmetricrequest import (
    UpdateSpanMetricRequest,
    UpdateSpanMetricRequestTypedDict,
)
from opperai.types import BaseModel
from opperai.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class UpdateMetricSpansSpanIDMetricsMetricIDPatchRequestTypedDict(TypedDict):
    span_id: str
    r"""The id of the span"""
    metric_id: str
    r"""The id of the metric to update"""
    update_span_metric_request: UpdateSpanMetricRequestTypedDict


class UpdateMetricSpansSpanIDMetricsMetricIDPatchRequest(BaseModel):
    span_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The id of the span"""

    metric_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The id of the metric to update"""

    update_span_metric_request: Annotated[
        UpdateSpanMetricRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .example import Example, ExampleTypedDict
from .functioncallconfiguration import (
    FunctionCallConfiguration,
    FunctionCallConfigurationTypedDict,
)
from .tmodel import TModel, TModelTypedDict
from opperai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class AppAPIPublicV2FunctionCallCallFunctionRequestTypedDict(TypedDict):
    name: str
    r"""Provide a unique name of the task. A function with this name will be created in the project. Functions configuration is overridden by the request parameters."""
    instructions: NotRequired[Nullable[str]]
    r"""Optionally provide an instruction for the model to complete the task. Recommended to be concise and to the point"""
    input_schema: NotRequired[Nullable[Dict[str, Any]]]
    r"""Optionally provide an input schema for the task. Can preferably include field descriptions to allow the model to reason about the input variables. Schema is validated against the input data and issues an error if it does not match. With the Opper SDKs you can define these schemas through libraries like Pydantic and Zod. For schemas with definitions, prefer using '$defs' and '#/$defs/...' references."""
    output_schema: NotRequired[Nullable[Dict[str, Any]]]
    r"""Optionally provide an output schema for the task. Response is guaranteed to match the schema or throw an error. Can preferably include field descriptions to allow the model to reason about the output variables. With the Opper SDKs you can define these schemas through libraries like Pydantic and Zod. For schemas with definitions, prefer using '$defs' and '#/$defs/...' references."""
    input: NotRequired[Nullable[Any]]
    r"""Optionally provide input data as context to complete the task. Could be a text, image, audio or a combination of these."""
    model: NotRequired[TModelTypedDict]
    examples: NotRequired[Nullable[List[ExampleTypedDict]]]
    r"""Optionally provide examples of successful task completions. Will be added to the prompt to help the model understand the task from examples."""
    parent_span_id: NotRequired[Nullable[str]]
    r"""Optionally provide the parent span ID to add to the call event. This will automatically tie the call to a parent span in the UI."""
    tags: NotRequired[Nullable[Dict[str, str]]]
    r"""Optionally provide a list of tags to add to the call event. Useful for being able to understand aggregate analytics on some dimension."""
    configuration: NotRequired[Nullable[FunctionCallConfigurationTypedDict]]
    r"""Optional configuration for the function.Configuration is a dictionary of key-value pairs that can be used to override the default configuration for the function."""


class AppAPIPublicV2FunctionCallCallFunctionRequest(BaseModel):
    name: str
    r"""Provide a unique name of the task. A function with this name will be created in the project. Functions configuration is overridden by the request parameters."""

    instructions: OptionalNullable[str] = UNSET
    r"""Optionally provide an instruction for the model to complete the task. Recommended to be concise and to the point"""

    input_schema: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Optionally provide an input schema for the task. Can preferably include field descriptions to allow the model to reason about the input variables. Schema is validated against the input data and issues an error if it does not match. With the Opper SDKs you can define these schemas through libraries like Pydantic and Zod. For schemas with definitions, prefer using '$defs' and '#/$defs/...' references."""

    output_schema: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Optionally provide an output schema for the task. Response is guaranteed to match the schema or throw an error. Can preferably include field descriptions to allow the model to reason about the output variables. With the Opper SDKs you can define these schemas through libraries like Pydantic and Zod. For schemas with definitions, prefer using '$defs' and '#/$defs/...' references."""

    input: OptionalNullable[Any] = UNSET
    r"""Optionally provide input data as context to complete the task. Could be a text, image, audio or a combination of these."""

    model: Optional[TModel] = None

    examples: OptionalNullable[List[Example]] = UNSET
    r"""Optionally provide examples of successful task completions. Will be added to the prompt to help the model understand the task from examples."""

    parent_span_id: OptionalNullable[str] = UNSET
    r"""Optionally provide the parent span ID to add to the call event. This will automatically tie the call to a parent span in the UI."""

    tags: OptionalNullable[Dict[str, str]] = UNSET
    r"""Optionally provide a list of tags to add to the call event. Useful for being able to understand aggregate analytics on some dimension."""

    configuration: OptionalNullable[FunctionCallConfiguration] = UNSET
    r"""Optional configuration for the function.Configuration is a dictionary of key-value pairs that can be used to override the default configuration for the function."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "instructions",
            "input_schema",
            "output_schema",
            "input",
            "model",
            "examples",
            "parent_span_id",
            "tags",
            "configuration",
        ]
        nullable_fields = [
            "instructions",
            "input_schema",
            "output_schema",
            "input",
            "examples",
            "parent_span_id",
            "tags",
            "configuration",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai.types import BaseModel
from opperai.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class GetFunctionByNameFunctionsByNameNameGetRequestTypedDict(TypedDict):
    name: str
    r"""The name of the function to retrieve"""


class GetFunctionByNameFunctionsByNameNameGetRequest(BaseModel):
    name: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The name of the function to retrieve"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcompletioncontentparttextparam import (
    ChatCompletionContentPartTextParam,
    ChatCompletionContentPartTextParamTypedDict,
)
from opperai.types import BaseModel
from opperai.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ChatCompletionDeveloperMessageParamContentTypedDict = TypeAliasType(
    "ChatCompletionDeveloperMessageParamContentTypedDict",
    Union[str, List[ChatCompletionContentPartTextParamTypedDict]],
)


ChatCompletionDeveloperMessageParamContent = TypeAliasType(
    "ChatCompletionDeveloperMessageParamContent",
    Union[str, List[ChatCompletionContentPartTextParam]],
)


class ChatCompletionDeveloperMessageParamTypedDict(TypedDict):
    content: ChatCompletionDeveloperMessageParamContentTypedDict
    role: Literal["developer"]
    name: NotRequired[str]


class ChatCompletionDeveloperMessageParam(BaseModel):
    content: ChatCompletionDeveloperMessageParamContent

    ROLE: Annotated[
        Annotated[Literal["developer"], AfterValidator(validate_const("developer"))],
        pydantic.Field(alias="role"),
    ] = "developer"

    name: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from opperai import errors, models, utils
from opperai._hooks import HookContext
from opperai.types import OptionalNullable, UNSET
from opperai.utils import get_security_from_env
from typing import Any, Mapping, Optional


class SpanMetrics(BaseSDK):
    def create_metric(
        self,
        *,
        span_id: str,
        dimension: str,
        value: float,
        comment: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateSpanMetricResponse:
        r"""Create Metric

        Create a new metric for a span

        :param span_id: The id of the span
        :param dimension: The dimension of the metric
        :param value: The value of the metric
        :param comment: A comment about the metric, e.g. a description of the metric
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateMetricSpansSpanIDMetricsPostRequest(
            span_id=span_id,
            create_span_metric_request=models.CreateSpanMetricRequest(
                dimension=dimension,
                value=value,
                comment=comment,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/spans/{span_id}/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_span_metric_request,
                False,
                False,
                "json",
                models.CreateSpanMetricRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_metric_spans__span_id__metrics_post",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CreateSpanMetricResponse)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.ConflictErrorData
            )
            raise errors.ConflictError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def create_metric_async(
        self,
        *,
        span_id: str,
        dimension: str,
        value: float,
        comment: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateSpanMetricResponse:
        r"""Create Metric

        Create a new metric for a span

        :param span_id: The id of the span
        :param dimension: The dimension of the metric
        :param value: The value of the metric
        :param comment: A comment about the metric, e.g. a description of the metric
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateMetricSpansSpanIDMetricsPostRequest(
            span_id=span_id,
            create_span_metric_request=models.CreateSpanMetricRequest(
                dimension=dimension,
                value=value,
                comment=comment,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/spans/{span_id}/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_span_metric_request,
                False,
                False,
                "json",
                models.CreateSpanMetricRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_metric_spans__span_id__metrics_post",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CreateSpanMetricResponse)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.ConflictErrorData
            )
            raise errors.ConflictError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def list(
        self,
        *,
        span_id: str,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PaginatedResponseListSpanMetricsResponse:
        r"""List Metrics

        List metrics for a span

        :param span_id: The id of the span to list metrics for
        :param offset: The offset to start the list from
        :param limit: The number of metrics to return
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListMetricsSpansSpanIDMetricsGetRequest(
            span_id=span_id,
            offset=offset,
            limit=limit,
        )

        req = self._build_request(
            method="GET",
            path="/spans/{span_id}/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list_metrics_spans__span_id__metrics_get",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.PaginatedResponseListSpanMetricsResponse
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def list_async(
        self,
        *,
        span_id: str,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PaginatedResponseListSpanMetricsResponse:
        r"""List Metrics

        List metrics for a span

        :param span_id: The id of the span to list metrics for
        :param offset: The offset to start the list from
        :param limit: The number of metrics to return
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListMetricsSpansSpanIDMetricsGetRequest(
            span_id=span_id,
            offset=offset,
            limit=limit,
        )

        req = self._build_request_async(
            method="GET",
            path="/spans/{span_id}/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list_metrics_spans__span_id__metrics_get",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.PaginatedResponseListSpanMetricsResponse
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get(
        self,
        *,
        span_id: str,
        metric_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetSpanMetricResponse:
        r"""Get Metric

        Get a metric for a span

        :param span_id: The id of the span
        :param metric_id: The id of the metric to get
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetMetricSpansSpanIDMetricsMetricIDGetRequest(
            span_id=span_id,
            metric_id=metric_id,
        )

        req = self._build_request(
            method="GET",
            path="/spans/{span_id}/metrics/{metric_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_metric_spans__span_id__metrics__metric_id__get",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.GetSpanMetricResponse)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        span_id: str,
        metric_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetSpanMetricResponse:
        r"""Get Metric

        Get a metric for a span

        :param span_id: The id of the span
        :param metric_id: The id of the metric to get
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetMetricSpansSpanIDMetricsMetricIDGetRequest(
            span_id=span_id,
            metric_id=metric_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/spans/{span_id}/metrics/{metric_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_metric_spans__span_id__metrics__metric_id__get",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.GetSpanMetricResponse)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def update_metric(
        self,
        *,
        span_id: str,
        metric_id: str,
        dimension: OptionalNullable[str] = UNSET,
        value: OptionalNullable[float] = UNSET,
        comment: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.UpdateSpanMetricResponse:
        r"""Update Metric

        Update a metric for a span

        :param span_id: The id of the span
        :param metric_id: The id of the metric to update
        :param dimension: The dimension of the metric
        :param value: The value of the metric
        :param comment: A comment about the metric, e.g. a description of the metric
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateMetricSpansSpanIDMetricsMetricIDPatchRequest(
            span_id=span_id,
            metric_id=metric_id,
            update_span_metric_request=models.UpdateSpanMetricRequest(
                dimension=dimension,
                value=value,
                comment=comment,
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/spans/{span_id}/metrics/{metric_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_span_metric_request,
                False,
                False,
                "json",
                models.UpdateSpanMetricRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_metric_spans__span_id__metrics__metric_id__patch",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.UpdateSpanMetricResponse)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def update_metric_async(
        self,
        *,
        span_id: str,
        metric_id: str,
        dimension: OptionalNullable[str] = UNSET,
        value: OptionalNullable[float] = UNSET,
        comment: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.UpdateSpanMetricResponse:
        r"""Update Metric

        Update a metric for a span

        :param span_id: The id of the span
        :param metric_id: The id of the metric to update
        :param dimension: The dimension of the metric
        :param value: The value of the metric
        :param comment: A comment about the metric, e.g. a description of the metric
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateMetricSpansSpanIDMetricsMetricIDPatchRequest(
            span_id=span_id,
            metric_id=metric_id,
            update_span_metric_request=models.UpdateSpanMetricRequest(
                dimension=dimension,
                value=value,
                comment=comment,
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/spans/{span_id}/metrics/{metric_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_span_metric_request,
                False,
                False,
                "json",
                models.UpdateSpanMetricRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_metric_spans__span_id__metrics__metric_id__patch",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.UpdateSpanMetricResponse)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def delete(
        self,
        *,
        span_id: str,
        metric_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete Metric

        Delete a metric for a span

        :param span_id:
        :param metric_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteMetricSpansSpanIDMetricsMetricIDDeleteRequest(
            span_id=span_id,
            metric_id=metric_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/spans/{span_id}/metrics/{metric_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="delete_metric_spans__span_id__metrics__metric_id__delete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def delete_async(
        self,
        *,
        span_id: str,
        metric_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete Metric

        Delete a metric for a span

        :param span_id:
        :param metric_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteMetricSpansSpanIDMetricsMetricIDDeleteRequest(
            span_id=span_id,
            metric_id=metric_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/spans/{span_id}/metrics/{metric_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="delete_metric_spans__span_id__metrics__metric_id__delete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.BadRequestErrorData
            )
            raise errors.BadRequestError(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.UnauthorizedErrorData
            )
            raise errors.UnauthorizedError(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.NotFoundErrorData
            )
            raise errors.NotFoundError(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.RequestValidationErrorData
            )
            raise errors.RequestValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai import utils
from opperai.types import BaseModel
from typing import Any


class ErrorData(BaseModel):
    type: str

    message: str

    detail: Any


class Error(Exception):
    data: ErrorData

    def __init__(self, data: ErrorData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, ErrorData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from opperai.types import BaseModel
import pydantic
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class ChatCompletionAudioParamFormat(str, Enum):
    WAV = "wav"
    AAC = "aac"
    MP3 = "mp3"
    FLAC = "flac"
    OPUS = "opus"
    PCM16 = "pcm16"


class VoiceEnum(str, Enum):
    ALLOY = "alloy"
    ASH = "ash"
    BALLAD = "ballad"
    CORAL = "coral"
    ECHO = "echo"
    FABLE = "fable"
    ONYX = "onyx"
    NOVA = "nova"
    SAGE = "sage"
    SHIMMER = "shimmer"
    VERSE = "verse"


VoiceTypedDict = TypeAliasType("VoiceTypedDict", Union[str, VoiceEnum])


Voice = TypeAliasType("Voice", Union[str, VoiceEnum])


class ChatCompletionAudioParamTypedDict(TypedDict):
    format_: ChatCompletionAudioParamFormat
    voice: VoiceTypedDict


class ChatCompletionAudioParam(BaseModel):
    format_: Annotated[ChatCompletionAudioParamFormat, pydantic.Field(alias="format")]

    voice: Voice



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .app_api_public_v2_functions_callfunctionrequest import (
    AppAPIPublicV2FunctionsCallFunctionRequest,
    AppAPIPublicV2FunctionsCallFunctionRequestTypedDict,
)
from opperai.types import BaseModel
from opperai.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class CallFunctionRevisionFunctionsFunctionIDCallRevisionIDPostRequestTypedDict(
    TypedDict
):
    function_id: str
    r"""The id of the function to call"""
    revision_id: str
    r"""The id of the revision to call"""
    app_api_public_v2_functions_call_function_request: (
        AppAPIPublicV2FunctionsCallFunctionRequestTypedDict
    )


class CallFunctionRevisionFunctionsFunctionIDCallRevisionIDPostRequest(BaseModel):
    function_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The id of the function to call"""

    revision_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The id of the revision to call"""

    app_api_public_v2_functions_call_function_request: Annotated[
        AppAPIPublicV2FunctionsCallFunctionRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List
from typing_extensions import NotRequired, TypedDict


class SpanDataTypedDict(TypedDict):
    input: NotRequired[Nullable[str]]
    r"""The input of the span"""
    output: NotRequired[Nullable[str]]
    r"""The output of the span"""
    total_tokens: NotRequired[Nullable[int]]
    r"""The total tokens of the span"""
    model: NotRequired[Nullable[str]]
    r"""The model of the span"""
    instructions: NotRequired[Nullable[str]]
    r"""The instructions of the span in case of a generation span"""
    function: NotRequired[Nullable[str]]
    r"""The function of the span"""
    tags: NotRequired[Nullable[List[str]]]
    r"""The tags of the span"""
    score: NotRequired[Nullable[int]]
    r"""The score of the span, by default set to the evaluation score of the generation of the span"""
    generation_id: NotRequired[Nullable[str]]
    r"""The id of the generation of the span"""
    observations: NotRequired[Nullable[str]]
    r"""The observations of the span in case of a generation span"""


class SpanData(BaseModel):
    input: OptionalNullable[str] = UNSET
    r"""The input of the span"""

    output: OptionalNullable[str] = UNSET
    r"""The output of the span"""

    total_tokens: OptionalNullable[int] = UNSET
    r"""The total tokens of the span"""

    model: OptionalNullable[str] = UNSET
    r"""The model of the span"""

    instructions: OptionalNullable[str] = UNSET
    r"""The instructions of the span in case of a generation span"""

    function: OptionalNullable[str] = UNSET
    r"""The function of the span"""

    tags: OptionalNullable[List[str]] = UNSET
    r"""The tags of the span"""

    score: OptionalNullable[int] = UNSET
    r"""The score of the span, by default set to the evaluation score of the generation of the span"""

    generation_id: OptionalNullable[str] = UNSET
    r"""The id of the generation of the span"""

    observations: OptionalNullable[str] = UNSET
    r"""The observations of the span in case of a generation span"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "input",
            "output",
            "total_tokens",
            "model",
            "instructions",
            "function",
            "tags",
            "score",
            "generation_id",
            "observations",
        ]
        nullable_fields = [
            "input",
            "output",
            "total_tokens",
            "model",
            "instructions",
            "function",
            "tags",
            "score",
            "generation_id",
            "observations",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from opperai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict
from typing_extensions import NotRequired, TypedDict


class GetSpanResponseTypedDict(TypedDict):
    name: str
    r"""The name of the span, something descriptive about the span that will be used to identify it when querying"""
    id: str
    start_time: NotRequired[Nullable[datetime]]
    r"""The start time of the span in UTC"""
    trace_id: NotRequired[Nullable[str]]
    r"""The id of the trace"""
    parent_id: NotRequired[Nullable[str]]
    r"""The id of the parent span"""
    type: NotRequired[Nullable[str]]
    r"""The type of the span"""
    end_time: NotRequired[Nullable[datetime]]
    r"""The end time of the span in UTC"""
    input: NotRequired[Nullable[Any]]
    r"""The input of the span"""
    output: NotRequired[Nullable[Any]]
    r"""The output of the span"""
    error: NotRequired[Nullable[str]]
    r"""In case of an error, the error message"""
    meta: NotRequired[Nullable[Dict[str, Any]]]
    r"""The metadata of the span"""
    score: NotRequired[Nullable[int]]
    r"""The score of the span"""


class GetSpanResponse(BaseModel):
    name: str
    r"""The name of the span, something descriptive about the span that will be used to identify it when querying"""

    id: str

    start_time: OptionalNullable[datetime] = UNSET
    r"""The start time of the span in UTC"""

    trace_id: OptionalNullable[str] = UNSET
    r"""The id of the trace"""

    parent_id: OptionalNullable[str] = UNSET
    r"""The id of the parent span"""

    type: OptionalNullable[str] = UNSET
    r"""The type of the span"""

    end_time: OptionalNullable[datetime] = UNSET
    r"""The end time of the span in UTC"""

    input: OptionalNullable[Any] = UNSET
    r"""The input of the span"""

    output: OptionalNullable[Any] = UNSET
    r"""The output of the span"""

    error: OptionalNullable[str] = UNSET
    r"""In case of an error, the error message"""

    meta: OptionalNullable[Dict[str, Any]] = UNSET
    r"""The metadata of the span"""

    score: OptionalNullable[int] = UNSET
    r"""The score of the span"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "start_time",
            "trace_id",
            "parent_id",
            "type",
            "end_time",
            "input",
            "output",
            "error",
            "meta",
            "score",
        ]
        nullable_fields = [
            "start_time",
            "trace_id",
            "parent_id",
            "type",
            "end_time",
            "input",
            "output",
            "error",
            "meta",
            "score",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .tmodel import TModel, TModelTypedDict
from opperai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


InputTypedDict = TypeAliasType("InputTypedDict", Union[str, List[str]])
r"""The input to embed, can be a single string or a list of strings"""


Input = TypeAliasType("Input", Union[str, List[str]])
r"""The input to embed, can be a single string or a list of strings"""


class CreateEmbeddingRequestTypedDict(TypedDict):
    input: InputTypedDict
    r"""The input to embed, can be a single string or a list of strings"""
    model: NotRequired[Nullable[TModelTypedDict]]
    r"""The model to use for the embedding, if not provided, `EmbeddingModel(hosting_provider='Azure', location='EU', identifier='azure/text-embedding-3-large', name='azure/text-embedding-3-large-1536', dimensions=1536, max_tokens=8191, credentials='azure-se', is_default=True, is_public=True, extra={'api_base': 'https://opperopenaiservice.openai.azure.com', 'api_version': '2023-05-15', 'dimensions': 1536})` will be used"""


class CreateEmbeddingRequest(BaseModel):
    input: Input
    r"""The input to embed, can be a single string or a list of strings"""

    model: OptionalNullable[TModel] = UNSET
    r"""The model to use for the embedding, if not provided, `EmbeddingModel(hosting_provider='Azure', location='EU', identifier='azure/text-embedding-3-large', name='azure/text-embedding-3-large-1536', dimensions=1536, max_tokens=8191, credentials='azure-se', is_default=True, is_public=True, extra={'api_base': 'https://opperopenaiservice.openai.azure.com', 'api_version': '2023-05-15', 'dimensions': 1536})` will be used"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["model"]
        nullable_fields = ["model"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai.types import BaseModel
from opperai.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class GetTraceTracesTraceIDGetRequestTypedDict(TypedDict):
    trace_id: str
    r"""The id of the trace to get"""


class GetTraceTracesTraceIDGetRequest(BaseModel):
    trace_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The id of the trace to get"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .audio import Audio, AudioTypedDict
from .chatcompletioncontentpartrefusalparam import (
    ChatCompletionContentPartRefusalParam,
    ChatCompletionContentPartRefusalParamTypedDict,
)
from .chatcompletioncontentparttextparam import (
    ChatCompletionContentPartTextParam,
    ChatCompletionContentPartTextParamTypedDict,
)
from .chatcompletionmessagetoolcallparam import (
    ChatCompletionMessageToolCallParam,
    ChatCompletionMessageToolCallParamTypedDict,
)
from .functioncall_input import FunctionCallInput, FunctionCallInputTypedDict
from opperai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from opperai.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ChatCompletionAssistantMessageParamContent1TypedDict = TypeAliasType(
    "ChatCompletionAssistantMessageParamContent1TypedDict",
    Union[
        ChatCompletionContentPartTextParamTypedDict,
        ChatCompletionContentPartRefusalParamTypedDict,
    ],
)


ChatCompletionAssistantMessageParamContent1 = TypeAliasType(
    "ChatCompletionAssistantMessageParamContent1",
    Union[ChatCompletionContentPartTextParam, ChatCompletionContentPartRefusalParam],
)


ChatCompletionAssistantMessageParamContent2TypedDict = TypeAliasType(
    "ChatCompletionAssistantMessageParamContent2TypedDict",
    Union[str, List[ChatCompletionAssistantMessageParamContent1TypedDict]],
)


ChatCompletionAssistantMessageParamContent2 = TypeAliasType(
    "ChatCompletionAssistantMessageParamContent2",
    Union[str, List[ChatCompletionAssistantMessageParamContent1]],
)


class ChatCompletionAssistantMessageParamTypedDict(TypedDict):
    role: Literal["assistant"]
    audio: NotRequired[Nullable[AudioTypedDict]]
    content: NotRequired[Nullable[ChatCompletionAssistantMessageParamContent2TypedDict]]
    function_call: NotRequired[Nullable[FunctionCallInputTypedDict]]
    name: NotRequired[str]
    refusal: NotRequired[Nullable[str]]
    tool_calls: NotRequired[List[ChatCompletionMessageToolCallParamTypedDict]]


class ChatCompletionAssistantMessageParam(BaseModel):
    ROLE: Annotated[
        Annotated[Literal["assistant"], AfterValidator(validate_const("assistant"))],
        pydantic.Field(alias="role"),
    ] = "assistant"

    audio: OptionalNullable[Audio] = UNSET

    content: OptionalNullable[ChatCompletionAssistantMessageParamContent2] = UNSET

    function_call: OptionalNullable[FunctionCallInput] = UNSET

    name: Optional[str] = None

    refusal: OptionalNullable[str] = UNSET

    tool_calls: Optional[List[ChatCompletionMessageToolCallParam]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "audio",
            "content",
            "function_call",
            "name",
            "refusal",
            "tool_calls",
        ]
        nullable_fields = ["audio", "content", "function_call", "refusal"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .updatecustommodelrequest import (
    UpdateCustomModelRequest,
    UpdateCustomModelRequestTypedDict,
)
from opperai.types import BaseModel
from opperai.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class UpdateCustomModelModelsCustomModelIDPatchRequestTypedDict(TypedDict):
    model_id: str
    r"""The ID of the custom language model to update"""
    update_custom_model_request: UpdateCustomModelRequestTypedDict


class UpdateCustomModelModelsCustomModelIDPatchRequest(BaseModel):
    model_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The ID of the custom language model to update"""

    update_custom_model_request: Annotated[
        UpdateCustomModelRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from opperai.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class Detail(str, Enum):
    AUTO = "auto"
    LOW = "low"
    HIGH = "high"


class ImageURLTypedDict(TypedDict):
    url: str
    detail: NotRequired[Detail]


class ImageURL(BaseModel):
    url: str

    detail: Optional[Detail] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openai_types_chat_chat_completion_message_tool_call_param_function import (
    OpenaiTypesChatChatCompletionMessageToolCallParamFunction,
    OpenaiTypesChatChatCompletionMessageToolCallParamFunctionTypedDict,
)
from opperai.types import BaseModel
from opperai.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class ChatCompletionMessageToolCallParamTypedDict(TypedDict):
    id: str
    function: OpenaiTypesChatChatCompletionMessageToolCallParamFunctionTypedDict
    type: Literal["function"]


class ChatCompletionMessageToolCallParam(BaseModel):
    id: str

    function: OpenaiTypesChatChatCompletionMessageToolCallParamFunction

    TYPE: Annotated[
        Annotated[Literal["function"], AfterValidator(validate_const("function"))],
        pydantic.Field(alias="type"),
    ] = "function"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createdatasetentryrequest import (
    CreateDatasetEntryRequest,
    CreateDatasetEntryRequestTypedDict,
)
from opperai.types import BaseModel
from opperai.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class CreateDatasetEntryDatasetsDatasetIDPostRequestTypedDict(TypedDict):
    dataset_id: str
    r"""The id of the dataset"""
    create_dataset_entry_request: CreateDatasetEntryRequestTypedDict


class CreateDatasetEntryDatasetsDatasetIDPostRequest(BaseModel):
    dataset_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The id of the dataset"""

    create_dataset_entry_request: Annotated[
        CreateDatasetEntryRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai.types import BaseModel
from opperai.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class QueryDatasetEntriesDatasetsDatasetIDEntriesQueryPostRequestTypedDict(TypedDict):
    dataset_id: str
    r"""The id of the dataset"""
    query: str
    r"""The query to search for"""
    limit: NotRequired[int]
    r"""The limit of the entries to get"""


class QueryDatasetEntriesDatasetsDatasetIDEntriesQueryPostRequest(BaseModel):
    dataset_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The id of the dataset"""

    query: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""The query to search for"""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 5
    r"""The limit of the entries to get"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FunctionCallConfigurationTypedDict(TypedDict):
    invocation_few_shot_count: NotRequired[int]
    r"""The number of few-shot examples to use for the call. The examples are selected using nearest neighbor search of the function's dataset for items that are similar to the input."""
    beta_evaluation_enabled: NotRequired[bool]
    r"""Whether to enable evaluation for the call. Evaluation is a beta feature and is enabled by default."""
    invocation_structured_generation_max_attempts: NotRequired[int]
    r"""The maximum number of attempts to make when generating a response matching the output schema if provided."""
    invocation_cache_ttl: NotRequired[int]
    r"""The time to live for the cache in seconds. If 0, the cache is disabled."""


class FunctionCallConfiguration(BaseModel):
    invocation_few_shot_count: Annotated[
        Optional[int], pydantic.Field(alias="invocation.few_shot.count")
    ] = 0
    r"""The number of few-shot examples to use for the call. The examples are selected using nearest neighbor search of the function's dataset for items that are similar to the input."""

    beta_evaluation_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="beta.evaluation.enabled")
    ] = True
    r"""Whether to enable evaluation for the call. Evaluation is a beta feature and is enabled by default."""

    invocation_structured_generation_max_attempts: Annotated[
        Optional[int],
        pydantic.Field(alias="invocation.structured_generation.max_attempts"),
    ] = 5
    r"""The maximum number of attempts to make when generating a response matching the output schema if provided."""

    invocation_cache_ttl: Annotated[
        Optional[int], pydantic.Field(alias="invocation.cache.ttl")
    ] = 0
    r"""The time to live for the cache in seconds. If 0, the cache is disabled."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from opperai.types import BaseModel
from typing_extensions import TypedDict


class CreateKnowledgeBaseResponseTypedDict(TypedDict):
    id: str
    name: str
    created_at: datetime
    embedding_model: str


class CreateKnowledgeBaseResponse(BaseModel):
    id: str

    name: str

    created_at: datetime

    embedding_model: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .addrequest import AddRequest, AddRequestTypedDict
from opperai.types import BaseModel
from opperai.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class AddKnowledgeKnowledgeBaseIDAddPostRequestTypedDict(TypedDict):
    knowledge_base_id: str
    r"""The id of the knowledge base to add the data to"""
    add_request: AddRequestTypedDict


class AddKnowledgeKnowledgeBaseIDAddPostRequest(BaseModel):
    knowledge_base_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The id of the knowledge base to add the data to"""

    add_request: Annotated[
        AddRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TextProcessingConfigurationTypedDict(TypedDict):
    text_processing_chunk_size: NotRequired[int]
    r"""The chunk size to use for the document. This is the number of characters to use for the chunk. The default is 2000 characters."""
    text_processing_chunk_overlap: NotRequired[int]
    r"""The chunk overlap to use for the document. This is the number of characters to overlap between chunks. The default is 200 characters."""


class TextProcessingConfiguration(BaseModel):
    text_processing_chunk_size: Annotated[
        Optional[int], pydantic.Field(alias="text_processing.chunk_size")
    ] = 2000
    r"""The chunk size to use for the document. This is the number of characters to use for the chunk. The default is 2000 characters."""

    text_processing_chunk_overlap: Annotated[
        Optional[int], pydantic.Field(alias="text_processing.chunk_overlap")
    ] = 200
    r"""The chunk overlap to use for the document. This is the number of characters to overlap between chunks. The default is 200 characters."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from opperai.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class ListFunctionsFunctionsGetRequestTypedDict(TypedDict):
    name: NotRequired[Nullable[str]]
    r"""Filter functions by name"""
    sort: NotRequired[Nullable[str]]
    r"""Sort the functions by name or created_at, use '-' to sort in descending order"""
    offset: NotRequired[Nullable[int]]
    r"""The offset of the page of functions to return when paginating"""
    limit: NotRequired[Nullable[int]]
    r"""The number of functions to return per page when paginating"""


class ListFunctionsFunctionsGetRequest(BaseModel):
    name: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter functions by name"""

    sort: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sort the functions by name or created_at, use '-' to sort in descending order"""

    offset: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The offset of the page of functions to return when paginating"""

    limit: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The number of functions to return per page when paginating"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["name", "sort", "offset", "limit"]
        nullable_fields = ["name", "sort", "offset", "limit"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .model import Model, ModelTypedDict
from typing import List, Union
from typing_extensions import TypeAliasType


TModel1TypedDict = TypeAliasType("TModel1TypedDict", Union[ModelTypedDict, str])


TModel1 = TypeAliasType("TModel1", Union[Model, str])


TModelTypedDict = TypeAliasType(
    "TModelTypedDict", Union[ModelTypedDict, str, List[TModel1TypedDict]]
)


TModel = TypeAliasType("TModel", Union[Model, str, List[TModel1]])



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, List, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


JSONPayloadTypedDict = TypeAliasType(
    "JSONPayloadTypedDict", Union[Dict[str, Any], List[Any], Any]
)
r"""Result of the task if the call uses an output schema"""


JSONPayload = TypeAliasType("JSONPayload", Union[Dict[str, Any], List[Any], Any])
r"""Result of the task if the call uses an output schema"""


class AppAPIPublicV2FunctionCallCallFunctionResponseTypedDict(TypedDict):
    span_id: str
    r"""The ID of the span of the call"""
    message: NotRequired[Nullable[str]]
    r"""Result of the task if the call does not use an output schema"""
    json_payload: NotRequired[Nullable[JSONPayloadTypedDict]]
    r"""Result of the task if the call uses an output schema"""
    audio: NotRequired[Nullable[Dict[str, Any]]]
    cached: NotRequired[Nullable[bool]]
    r"""True if the result was returned from a cached results"""
    images: NotRequired[Nullable[List[str]]]
    r"""The images generated by the call. Only available for image models. Depending on the configuration, the response can either be a list of image urls or a base64 encoded images."""


class AppAPIPublicV2FunctionCallCallFunctionResponse(BaseModel):
    span_id: str
    r"""The ID of the span of the call"""

    message: OptionalNullable[str] = UNSET
    r"""Result of the task if the call does not use an output schema"""

    json_payload: OptionalNullable[JSONPayload] = UNSET
    r"""Result of the task if the call uses an output schema"""

    audio: OptionalNullable[Dict[str, Any]] = UNSET

    cached: OptionalNullable[bool] = UNSET
    r"""True if the result was returned from a cached results"""

    images: OptionalNullable[List[str]] = UNSET
    r"""The images generated by the call. Only available for image models. Depending on the configuration, the response can either be a list of image urls or a base64 encoded images."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["message", "json_payload", "audio", "cached", "images"]
        nullable_fields = ["message", "json_payload", "audio", "cached", "images"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai.types import BaseModel
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class OpenaiTypesChatCompletionCreateParamsFunctionTypedDict(TypedDict):
    name: str
    description: NotRequired[str]
    parameters: NotRequired[Dict[str, Any]]


class OpenaiTypesChatCompletionCreateParamsFunction(BaseModel):
    name: str

    description: Optional[str] = None

    parameters: Optional[Dict[str, Any]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai import utils
from opperai.types import BaseModel
from typing import Any, Optional


class RequestValidationErrorData(BaseModel):
    detail: Any
    r"""Where the error occurred and the error message"""

    type: Optional[str] = "RequestValidationError"

    message: Optional[str] = "The request is invalid"


class RequestValidationError(Exception):
    data: RequestValidationErrorData

    def __init__(self, data: RequestValidationErrorData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, RequestValidationErrorData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .updatefunctionrequest import UpdateFunctionRequest, UpdateFunctionRequestTypedDict
from opperai.types import BaseModel
from opperai.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class UpdateFunctionFunctionsFunctionIDPatchRequestTypedDict(TypedDict):
    function_id: str
    r"""Unique identifier of the function given as a UUID"""
    update_function_request: UpdateFunctionRequestTypedDict


class UpdateFunctionFunctionsFunctionIDPatchRequest(BaseModel):
    function_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Unique identifier of the function given as a UUID"""

    update_function_request: Annotated[
        UpdateFunctionRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .functioncallconfiguration import (
    FunctionCallConfiguration,
    FunctionCallConfigurationTypedDict,
)
from .tmodel import TModel, TModelTypedDict
from opperai.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class UpdateFunctionResponseTypedDict(TypedDict):
    name: str
    r"""The name of the function. Must be unique within the project and can only contain letters, numbers, underscores and hyphens."""
    instructions: str
    r"""The instructions for the function, this is the prompt that will be sent to the model to complete the task. Recommended to be concise and to the point"""
    id: str
    r"""The ID of the function"""
    description: NotRequired[Nullable[str]]
    r"""Optional description of the function. This is used to describe the function to a user."""
    input_schema: NotRequired[Nullable[Dict[str, Any]]]
    r"""Optional input schema for the function. Can preferably include field descriptions to allow the model to reason about the input variables. Schema is validated against the input data and issues an error if it does not match. With the Opper SDKs you can define these schemas through libraries like Pydantic and Zod. For schemas with definitions, prefer using '$defs' and '#/$defs/...' references."""
    output_schema: NotRequired[Nullable[Dict[str, Any]]]
    r"""Optional output schema for the function. Can preferably include field descriptions to allow the model to reason about the output variables. Schema is validated against the output data and issues an error if it does not match. With the Opper SDKs you can define these schemas through libraries like Pydantic and Zod. For schemas with definitions, prefer using '$defs' and '#/$defs/...' references."""
    model: NotRequired[TModelTypedDict]
    configuration: NotRequired[Nullable[FunctionCallConfigurationTypedDict]]
    r"""Optional configuration for the function.Configuration is a dictionary of key-value pairs that can be used to override the default configuration for the function."""
    dataset_id: NotRequired[Nullable[str]]
    r"""The ID of the dataset associated with the function"""
    revision_id: NotRequired[Nullable[str]]
    r"""The ID of the latest revision of the function"""


class UpdateFunctionResponse(BaseModel):
    name: str
    r"""The name of the function. Must be unique within the project and can only contain letters, numbers, underscores and hyphens."""

    instructions: str
    r"""The instructions for the function, this is the prompt that will be sent to the model to complete the task. Recommended to be concise and to the point"""

    id: str
    r"""The ID of the function"""

    description: OptionalNullable[str] = UNSET
    r"""Optional description of the function. This is used to describe the function to a user."""

    input_schema: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Optional input schema for the function. Can preferably include field descriptions to allow the model to reason about the input variables. Schema is validated against the input data and issues an error if it does not match. With the Opper SDKs you can define these schemas through libraries like Pydantic and Zod. For schemas with definitions, prefer using '$defs' and '#/$defs/...' references."""

    output_schema: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Optional output schema for the function. Can preferably include field descriptions to allow the model to reason about the output variables. Schema is validated against the output data and issues an error if it does not match. With the Opper SDKs you can define these schemas through libraries like Pydantic and Zod. For schemas with definitions, prefer using '$defs' and '#/$defs/...' references."""

    model: Optional[TModel] = None

    configuration: OptionalNullable[FunctionCallConfiguration] = UNSET
    r"""Optional configuration for the function.Configuration is a dictionary of key-value pairs that can be used to override the default configuration for the function."""

    dataset_id: OptionalNullable[str] = UNSET
    r"""The ID of the dataset associated with the function"""

    revision_id: OptionalNullable[str] = UNSET
    r"""The ID of the latest revision of the function"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "description",
            "input_schema",
            "output_schema",
            "model",
            "configuration",
            "dataset_id",
            "revision_id",
        ]
        nullable_fields = [
            "description",
            "input_schema",
            "output_schema",
            "configuration",
            "dataset_id",
            "revision_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai.types import BaseModel
from opperai.utils import eventstreaming
from pydantic import SkipValidation
from typing import Dict, List, Optional, Union
from typing_extensions import NotRequired, TypedDict


class FunctionStreamCallStreamPostDataTypedDict(TypedDict):
    r"""The actual data payload containing streaming chunk information"""

    delta: NotRequired[str]
    r"""Incremental text content generated by the model"""
    span_id: NotRequired[str]
    r"""Unique identifier for the execution span"""


class FunctionStreamCallStreamPostData(BaseModel):
    r"""The actual data payload containing streaming chunk information"""

    delta: Optional[str] = None
    r"""Incremental text content generated by the model"""

    span_id: Optional[str] = None
    r"""Unique identifier for the execution span"""


class FunctionStreamCallStreamPostResponseBodyTypedDict(TypedDict):
    r"""Server-Sent Event following the SSE specification"""

    data: FunctionStreamCallStreamPostDataTypedDict
    r"""The actual data payload containing streaming chunk information"""
    id: NotRequired[str]
    r"""Event ID for the SSE event"""
    event: NotRequired[str]
    r"""Event type for the SSE event"""
    retry: NotRequired[int]
    r"""Retry interval in milliseconds for the SSE connection"""


class FunctionStreamCallStreamPostResponseBody(BaseModel):
    r"""Server-Sent Event following the SSE specification"""

    data: FunctionStreamCallStreamPostData
    r"""The actual data payload containing streaming chunk information"""

    id: Optional[str] = None
    r"""Event ID for the SSE event"""

    event: Optional[str] = None
    r"""Event type for the SSE event"""

    retry: Optional[int] = None
    r"""Retry interval in milliseconds for the SSE connection"""


class FunctionStreamCallStreamPostResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: Union[
        eventstreaming.EventStream[FunctionStreamCallStreamPostResponseBodyTypedDict],
        eventstreaming.EventStreamAsync[
            FunctionStreamCallStreamPostResponseBodyTypedDict
        ],
    ]


class FunctionStreamCallStreamPostResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: SkipValidation[
        Union[
            eventstreaming.EventStream[FunctionStreamCallStreamPostResponseBody],
            eventstreaming.EventStreamAsync[FunctionStreamCallStreamPostResponseBody],
        ]
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcompletioncontentpartimageparam import (
    ChatCompletionContentPartImageParam,
    ChatCompletionContentPartImageParamTypedDict,
)
from .chatcompletioncontentpartinputaudioparam import (
    ChatCompletionContentPartInputAudioParam,
    ChatCompletionContentPartInputAudioParamTypedDict,
)
from .chatcompletioncontentparttextparam import (
    ChatCompletionContentPartTextParam,
    ChatCompletionContentPartTextParamTypedDict,
)
from .file import File, FileTypedDict
from opperai.types import BaseModel
from opperai.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ChatCompletionUserMessageParamContent1TypedDict = TypeAliasType(
    "ChatCompletionUserMessageParamContent1TypedDict",
    Union[
        ChatCompletionContentPartTextParamTypedDict,
        ChatCompletionContentPartImageParamTypedDict,
        ChatCompletionContentPartInputAudioParamTypedDict,
        FileTypedDict,
    ],
)


ChatCompletionUserMessageParamContent1 = TypeAliasType(
    "ChatCompletionUserMessageParamContent1",
    Union[
        ChatCompletionContentPartTextParam,
        ChatCompletionContentPartImageParam,
        ChatCompletionContentPartInputAudioParam,
        File,
    ],
)


ChatCompletionUserMessageParamContent2TypedDict = TypeAliasType(
    "ChatCompletionUserMessageParamContent2TypedDict",
    Union[str, List[ChatCompletionUserMessageParamContent1TypedDict]],
)


ChatCompletionUserMessageParamContent2 = TypeAliasType(
    "ChatCompletionUserMessageParamContent2",
    Union[str, List[ChatCompletionUserMessageParamContent1]],
)


class ChatCompletionUserMessageParamTypedDict(TypedDict):
    content: ChatCompletionUserMessageParamContent2TypedDict
    role: Literal["user"]
    name: NotRequired[str]


class ChatCompletionUserMessageParam(BaseModel):
    content: ChatCompletionUserMessageParamContent2

    ROLE: Annotated[
        Annotated[Literal["user"], AfterValidator(validate_const("user"))],
        pydantic.Field(alias="role"),
    ] = "user"

    name: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from opperai.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class WebSearchOptionsUserLocationApproximateTypedDict(TypedDict):
    city: NotRequired[str]
    country: NotRequired[str]
    region: NotRequired[str]
    timezone: NotRequired[str]


class WebSearchOptionsUserLocationApproximate(BaseModel):
    city: Optional[str] = None

    country: Optional[str] = None

    region: Optional[str] = None

    timezone: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import TYPE_CHECKING
from importlib import import_module

if TYPE_CHECKING:
    from .apierror import APIError
    from .badrequesterror import BadRequestError, BadRequestErrorData
    from .conflicterror import ConflictError, ConflictErrorData
    from .error import Error, ErrorData
    from .notfounderror import NotFoundError, NotFoundErrorData
    from .requestvalidationerror import (
        RequestValidationError,
        RequestValidationErrorData,
    )
    from .unauthorizederror import UnauthorizedError, UnauthorizedErrorData

__all__ = [
    "APIError",
    "BadRequestError",
    "BadRequestErrorData",
    "ConflictError",
    "ConflictErrorData",
    "Error",
    "ErrorData",
    "NotFoundError",
    "NotFoundErrorData",
    "RequestValidationError",
    "RequestValidationErrorData",
    "UnauthorizedError",
    "UnauthorizedErrorData",
]

_dynamic_imports: dict[str, str] = {
    "APIError": ".apierror",
    "BadRequestError": ".badrequesterror",
    "BadRequestErrorData": ".badrequesterror",
    "ConflictError": ".conflicterror",
    "ConflictErrorData": ".conflicterror",
    "Error": ".error",
    "ErrorData": ".error",
    "NotFoundError": ".notfounderror",
    "NotFoundErrorData": ".notfounderror",
    "RequestValidationError": ".requestvalidationerror",
    "RequestValidationErrorData": ".requestvalidationerror",
    "UnauthorizedError": ".unauthorizederror",
    "UnauthorizedErrorData": ".unauthorizederror",
}


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)
